Package: plyr
Version: 1.8.3
Title: Tools for Splitting, Applying and Combining Data
Description: A set of tools that solves a common set of problems: you
    need to break a big problem down into manageable pieces, operate on each
    piece and then put all the pieces back together.  For example, you might
    want to fit a model to each spatial location or time point in your study,
    summarise data by panels or collapse high-dimensional arrays to simpler
    summary statistics. The development of 'plyr' has been generously supported
    by 'Becton Dickinson'.
Authors@R: person("Hadley", "Wickham", , "hadley@rstudio.com", c("aut", "cre"))
URL: http://had.co.nz/plyr, https://github.com/hadley/plyr
BugReports: https://github.com/hadley/plyr/issues
Depends: R (>= 3.1.0)
Imports: Rcpp (>= 0.11.0)
LinkingTo: Rcpp
Suggests: abind, testthat, tcltk, foreach, doParallel, itertools,
        iterators
License: MIT + file LICENSE
LazyData: true
NeedsCompilation: yes
Packaged: 2015-06-11 13:04:58 UTC; hadley
Author: Hadley Wickham [aut, cre]
Maintainer: Hadley Wickham <hadley@rstudio.com>
Repository: CRAN
Date/Publication: 2015-06-12 11:05:51
