#' @section Input: This function splits lists by elements and combines the
#'   result into a data frame.
#'
#' @param .data list to be processed
#' @param .parallel if \code{TRUE}, apply function in parallel, using parallel 
#'   backend provided by foreach
#' @family list input
