% Generated by roxygen2 (4.1.1): do not edit by hand
% Please edit documentation in R/utils.r
\name{is.formula}
\alias{is.formula}
\title{Is a formula?
Checks if argument is a formula}
\usage{
is.formula(x)
}
\description{
Is a formula?
Checks if argument is a formula
}
\keyword{internal}

