% Generated by roxygen2 (4.1.1): do not edit by hand
% Please edit documentation in R/utils.r
\name{compact}
\alias{compact}
\title{Compact list.}
\usage{
compact(l)
}
\arguments{
\item{l}{list}
}
\description{
Remove all NULL entries from a list
}
\keyword{internal}
\keyword{manip}

